@@ -31,10 +31,15 @@ from datetime import datetime
 import dxpy, dxpy.app_builder
 from dxpy import logger
 
+# Use default locale (used for formatting numbers nicely)
+locale.setlocale(locale.LC_ALL, '')
+
 from dxpy.utils.resolver import resolve_path, is_container_id
 
 parser = argparse.ArgumentParser(description="Uploads a DNAnexus App.")
 
+APP_VERSION_RE = re.compile("^([1-9][0-9]*|0)\.([1-9][0-9]*|0)\.([1-9][0-9]*|0)(-[-0-9A-Za-z]+(\.[-0-9A-Za-z]+)*)?(\+[-0-9A-Za-z]+(\.[-0-9A-Za-z]+)*)?$")
+
 # COMMON OPTIONS
 parser.add_argument("src_dir", help="App or applet source directory (default: current directory)", nargs='?')
 
@@ -125,18 +130,30 @@ parser.add_argument("--dry-run", "-n", help="Do not create an app(let): only per
 parser.add_argument("--no-dry-run", help=argparse.SUPPRESS, action="store_false", dest="dry_run")
 
 
-def get_timestamp_version_suffix():
-    return "+build." + datetime.today().strftime('%Y%m%d.%H%M')
+def _get_timestamp_version_suffix(version):
+    if "+" in version:
+        return ".build." + datetime.today().strftime('%Y%m%d.%H%M')
+    else:
+        return "+build." + datetime.today().strftime('%Y%m%d.%H%M')
 
-def get_version_suffix(src_dir):
+def _get_version_suffix(src_dir, version):
     # If anything goes wrong, fall back to the date-based suffix.
     try:
         if os.path.exists(os.path.join(src_dir, ".git")):
             abbrev_sha1 = subprocess.check_output(['git', 'rev-parse', 'HEAD'], cwd=src_dir).strip()[:7]
-            return "+git." + abbrev_sha1
+            # We ensure that if VERSION is semver-compliant, then
+            # VERSION + SUFFIX will be too. In particular that means
+            # (here and in _get_timestamp_version_suffix above) we add
+            # what semver refers to as a "build metadata" section
+            # (delimited by "+"), unless one already exists, in which
+            # case we append to the existing one.
+            if "+" in version:
+                return ".git." + abbrev_sha1
+            else:
+                return "+git." + abbrev_sha1
     except:
         pass
-    return get_timestamp_version_suffix()
+    return _get_timestamp_version_suffix(version)
 
 def parse_destination(dest_str):
     """
@@ -201,8 +218,8 @@ def _lint(dxapp_json_filename):
             logger.warn("app is missing a description, please supply one in README.md")
 
     if 'version' in app_spec:
-        if not re.match("^([1-9][0-9]*|0)\.([1-9][0-9]*|0)\.([1-9][0-9]*|0)$", app_spec['version']):
-            logger.warn('"version" %s should be of the form X.Y.Z' % (app_spec['version'],))
+        if not APP_VERSION_RE.match(app_spec['version']):
+            logger.warn('"version" %s should be semver compliant (e.g. of the form X.Y.Z)' % (app_spec['version'],))
 
     if 'categories' in app_spec:
         for category in app_spec['categories']:
@@ -390,7 +407,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                 if app_describe.has_key('published') or not do_try_update:
                     # The version we wanted was taken; fall back to the
                     # autogenerated version number.
-                    build_options['version_override'] = original_version + get_version_suffix(src_dir)
+                    build_options['version_override'] = original_version + _get_version_suffix(src_dir, original_version)
 
     # The following flags are basically passed through verbatim.
     if bill_to_override:
@@ -583,7 +600,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, publish=False, dest
             version = app_json['version']
             try_versions = [version_override or version]
             if not version_override and do_version_autonumbering:
-                try_versions.append(version + get_version_suffix(src_dir))
+                try_versions.append(version + _get_version_suffix(src_dir, version))
 
             app_id = dxpy.app_builder.create_app(applet_id,
                                                  applet_name,
